check_public_skip_exponent_size

Function check_public_skip_exponent_size 

fn check_public_skip_exponent_size(n: &BoxedUint, e: &BoxedUint) -> Result<()>
Check that the public key is well formed, skipping exponent size bounds checks.

This is used internally by both public validation functions and hazmat APIs.